# -*- coding: iso-8859-15 -*-
# arty.py
# 5/9/06
# --- kiff: kpbayne@elp.rr.com

import host 
import new 
import bf2 
import math 
from bf2 import g_debug


FAST_TIMER_PERIOD = 0.1
NORMAL_TIMER_PERIOD = 0.5
CHECK_NEWCAM_COUNT = 5

HOLD_LAST_TRACKING_POSITION_COUNT = 6

SPOT_TIME = 15.0
SPOT_TIME_COUNT = 6

gCheckNewCamRemaining = CHECK_NEWCAM_COUNT
gTrackingActive = 0

gSpots = [{},{},{}]
gCams = [{},{},{}]
gLastPlayersSpotters = {}
gPlayersInArty = [{},{},{}]
gCamTimer = None

ZERO_VECT3 = (0.0,0.0,0.0)

# g_debug = 1


ARTY_VEHICLES =	{
	'dcon_MLRS_operator'	: 'mobile_arty_camera_projectile_for_rockets',
	'dcon_bm21_operator'	: 'mobile_arty_camera_projectile_for_rockets',
	'dcon_m109v0_operator'	: 'mobile_arty_camera_projectile_for_guns',
	'iraqart_2s1_gunner'	: 'mobile_arty_camera_projectile_for_guns',
}


CAM_TEMPLATES = {
	'mobile_arty_camera_projectile_for_rockets'	: 'mobile_arty_rocket_projectile',
	'mobile_arty_camera_projectile_for_guns' 	: 'mobile_arty_gun_projectile',
}

NULL_CAMS = {
	'dcon_MLRS_operator'	: ('mlrs_tv_Launcher','mobile_arty_nocam_projectile'),
	'dcon_bm21_operator'	: ('tv_bm21_Launcher','mobile_arty_nocam_projectile'),
	'dcon_m109v0_operator'	: ('tv_dcon_m109v0_Launcher', 'mobile_arty_nocam_projectile'), 
	'iraqart_2s1_gunner'	: ('tv_dcon_m109v1_Launcher', 'mobile_arty_nocam_projectile'), 
}


class Arty_Spotter:
	def __init__(self,pos,rot,spotTimer,wtm):
		self.position = pos
		self.rotation = rot
		self.timer = spotTimer
		self.wallTime = wtm
		self.checksRemaining = SPOT_TIME_COUNT



class Arty_Camera:
	def __init__(self,camProj,spotId,spotPos):
		self.camProj = camProj
		self.spotterID = spotId
		self.trackingProj = None
		self.spotOrientation = spotPos #tuple = (pos,rot)
		self.holdLastTrackSpotCount = 0 #HOLD_LAST_TRACKING_POSITION_COUNT
		self.dirty = 1
		self.applyDefaultOrientation()	

	def applyDefaultOrientation(self):
		if self.dirty:# dont waste bandwidth
			self.dirty = 0
			self.camProj.setPosition(self.spotOrientation[0])
			self.camProj.setRotation(self.spotOrientation[1])

	def applyDefaultOrientationWithHold(self):
		if self.holdLastTrackSpotCount > 0:
			self.holdLastTrackSpotCount -= 1
		else:
			self.applyDefaultOrientation()

	def pointAtTrackedProj(self):
		if not self.trackingProj.isValid():
			return 0
		trkPos = self.trackingProj.getPosition()
		if trkPos == ZERO_VECT3:
			return 0
		vect = makeVect(self.spotOrientation[0],trkPos)
		yaw = math.degrees(math.atan2 (vect[0],vect[2]) )
		pitch = math.degrees(math.asin(vect[1] / vectMag(vect)) * -1.0)  
		self.camProj.setRotation((yaw,pitch,0.0))
		self.dirty = 1
		return 1





def init():
	host.registerGameStatusHandler(onGameStatusChangedARTY)
	if g_debug: print 'arty initialized'
	reset()
	bf2.Timer.__del__ = replace__del__


def replace__del__(self):
	# avoid flooding null stream, -seems- to cause lag :/
	pass #	print "timer object destroyed (rc 0)"	



def onGameStatusChangedARTY(status):
	reset()

	if status == bf2.GameStatus.Playing:
		# if g_debug:
			# host.registerHandler('ChatMessage', onChatMessageARTY)
		host.registerHandler('EnterVehicle', onEnterVehicleARTY)
		host.registerHandler('ExitVehicle', onExitVehicleARTY)
		host.registerHandler('RemoteCommand', onRemoteCommandARTY)

		# bah, let the arty guys keep the spot
		#host.registerHandler('PlayerDisconnect', onPlayerDisconnectARTY)
		#host.registerHandler('PlayerDeath',onPlayerDeathARTY)

		# might add spotter points later...
		#host.registerHandler('PlayerKilled',onPlayerKilledARTY)
		#host.registerHandler('VehicleDestroyed', onVehicleDestroyedARTY)


 

def reset():
	global gCams
	global gSpots
	global gLastPlayersSpotters
	global gPlayersInArty
	global gTrackingActive
	global CHECK_NEWCAM_COUNT
	global gCheckNewCamRemaining

	#print '**************** arty reset ********************'
	killArtyTimer()

	gCams = [{},{},{}]

	for t in (1,2):
		try:
			for spt in gSpots[t].values():
				try: spt.timer.destroy()
				except: pass
				spt.timer = None
		except:
			#print 'for spt in gSpots[t].values():'
			pass

	gSpots = [{},{},{}]

	gLastPlayersSpotters = {}
	gPlayersInArty = [{},{},{}]

	gTrackingActive = 0
	try: CHECK_NEWCAM_COUNT = int(math.ceil(NORMAL_TIMER_PERIOD / FAST_TIMER_PERIOD))
	except: pass
	if CHECK_NEWCAM_COUNT < 1:
		CHECK_NEWCAM_COUNT = 1

	gCheckNewCamRemaining = CHECK_NEWCAM_COUNT		



# #################################################	
# ################# Spotter Stuff ########################
# #################################################	



def newSpot(pos,rot,pid,team,pName):
	global gSpots


	spotTimer = bf2.Timer(spotTimeOut, SPOT_TIME, 1, (pid,team,pName))
	if SPOT_TIME_COUNT > 1:
		spotTimer.setRecurring(SPOT_TIME)

	pos = (pos[0], pos[1] + 20.0, pos[2])
	rot = (rot[0], rot[1] + 20.0, rot[2])
	wtm = host.timer_getWallTime() 
	newSptt = Arty_Spotter(pos,rot,spotTimer,wtm)

	if pid in gSpots[team]:
		removeCamsUsingSpot(team,pid)
		s = gSpots[team][pid]
		try: s.timer.destroy()
		except: pass
		s.timer = None
		gSpots[team][pid] = newSptt # update existing spot
		return
		
	oldSpotCount = len(gSpots[team])
	gSpots[team][pid] = newSptt
	
	if oldSpotCount == 0: # if there were no spots before now, see if we need to activate
		checkArtyCamsEnabled(team)
		checkArtyTimer()#should only need to check if we got here


def spotTimeOut(data):
	pid = data[0]
	team = data[1]
	sptrName = data[2]
	
	msg = 'game.sayTeam ' + str(team) + ' "C1001Artillery spot called by ' + sptrName + ' has '
	removed = 0
	if pid in gSpots[team]:
		s = gSpots[team][pid]
		s.checksRemaining -= 1
		if s.checksRemaining > 0:
			tmrmn = int(math.ceil(s.checksRemaining * SPOT_TIME))
			msg += str(tmrmn) + ' seconds remaining."'
		else:
			msg += ' timed out."'
			removed = 1
			try: s.timer.destroy()
			except: pass
			s.timer = None
			del gSpots[team][pid]
	else:
		if g_debug:
			print 'spotTimeOut: spotter not in dictionary'
		return
		

	host.rcon_invoke(msg)

	if not removed: return
	
	#kick out arty ppl using this cam
	removeCamsUsingSpot(team,pid)
	checkArtyCamsEnabled(team)
	checkArtyTimer()#should only need to check if we got here




def playerNextSpotter(pid):
	global gCams
	global gSpots
	p = bf2.playerManager.getPlayerByIndex(pid)
	t = p.getTeam()
	
	if len(gSpots[t]) < 1:
		return #  remove cam ?
	
	oldSpotId = -1
	if pid in gLastPlayersSpotters.keys():
		oldSpotId = gLastPlayersSpotters[pid]
	

	#assume wrap around to list start
	sptKeys = gSpots[t].keys()
	newSpotId = sptKeys[0]
	keysLen = len(sptKeys)
	for skind in range(keysLen):
		if sptKeys[skind] == oldSpotId: # then get the one after it
			if keysLen > (skind + 1):
				newSpotId = sptKeys[skind + 1]
				break

				
	if pid in gCams[t]:
		c = gCams[t][pid]
		spt = getSpot(newSpotId,t)
		spotPos = (spt.position,spt.rotation)
		c.spotterID = newSpotId
		c.spotOrientation = spotPos
		c.dirty = 1
		c.applyDefaultOrientation()
	#else: # no cam
	gLastPlayersSpotters[pid] = newSpotId # still select even when not in cam mode


def getBestSpotterIdForTeam(team):

	if len(gSpots[team]) < 1:
		return -1
	else:
		spotKeys = gSpots[team].keys()
		bestID = spotKeys[0] #assume first one is best
		bestTime = gSpots[team][bestID]
		for sk in spotKeys:
			sp = gSpots[team][sk]
			if sp.wallTime > bestTime: #found a better one
				bestTime = sp.wallTime
				bestID = sk

		return bestID


def getSpot(spotterId,team):
	if spotterId == -1: return None

	if spotterId in gSpots[team]:
		return gSpots[team][spotterId]

	return None


# #################################################
# ##################### Arty Cams  #####################
# #################################################


def playerEnterCam(pid,camProj,team,projList):

	global gLastPlayersSpotters


	spotId = -1
	nspt = None
	if pid in gLastPlayersSpotters:
		spt = gLastPlayersSpotters[pid]
		nspt = getSpot(spt,team)
		if nspt:
			spotId = spt

	if spotId == -1:
		spotId = getBestSpotterIdForTeam(team)

	if spotId == -1:
		playerExitCam(pid,camProj.templateName,team,projList)
		return None

	if not nspt:
		nspt = getSpot(spotId,team)
	spotPos = (nspt.position,nspt.rotation)

	newCm = Arty_Camera(camProj,spotId,spotPos) # no projectile to lookat

	gCams[team][pid] = newCm
	gLastPlayersSpotters[pid] = spotId
	return newCm

	
def playerExitCam(pid,camProj,team,projList):
	#print 'playerExitCam'
	if pid in gCams[team]:
		del gCams[team][pid]

	killProjectilesOfTemplate(camProj,projList)#.templateName)


def removeCamsUsingSpot(team,spotterID):
	found = 1 
	while found: #keep removing them
		found = 0
		for pid in gCams[team].keys():
			if gCams[team][pid].spotterID == spotterID:
				found = 1
				playerExitCam(pid,gCams[team][pid].camProj.templateName,team,None)
				break



# #################################################
# ##################### Main Loop  #####################
# #################################################


def camTimerHandler(data):
	#print 'camTimerHandler - TOP--------------'
	global gCams
	global gCheckNewCamRemaining
	global gTrackingActive 
	global gCamTimer

	oldTracking = gTrackingActive
	gTrackingActive = 0

	# look for new cams or proj's to track?
	if oldTracking:
		gCheckNewCamRemaining -= 1
		if gCheckNewCamRemaining < 1:
			checkForNewProjectiles()
			gCheckNewCamRemaining = CHECK_NEWCAM_COUNT
	else:
		checkForNewProjectiles()

	#update existing cams
	for t in (1,2):
		#print 'cams check t: ' + str(t)
		for cur in gCams[t].values():
			if cur.trackingProj is None and not cur.dirty:
				continue

			resortDefault = 1
			try: # hate to just try, but this is bf2 and there always seems to be another exception...
				if cur.trackingProj: # then we have an active projectile to track
					if cur.pointAtTrackedProj():
						gTrackingActive = 1
						resortDefault = 0
			except:
				if g_debug: 
					print 'What happened?'

			if resortDefault:	
				cur.applyDefaultOrientationWithHold()

	# adjust timer speed if necessary		
	if oldTracking != gTrackingActive:
		if gTrackingActive:
			gCamTimer.setRecurring(FAST_TIMER_PERIOD)
		else:
			gCamTimer.setRecurring(NORMAL_TIMER_PERIOD)
			


def checkForNewProjectiles():
	global gTrackingActive
	global gCams

	# cache projectile list. getObjectsOfTemplate is slower than getting all projectiles and using a python filter, go figure...
	allProjs = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.GenericProjectile')
	if len(allProjs) < 1:
		return # nothing to see here, move along..

	#print allProjs

	for pteam in (1,2):
		for pid in gPlayersInArty[pteam].keys():
			vehTemp = gPlayersInArty[pteam][pid]
			projTemp = ARTY_VEHICLES[vehTemp]
			proj = []
			for checkProj in allProjs:
				if checkProj.templateName == projTemp:
					proj.append(checkProj)

			if len(proj) < 1:
				continue

			found = 0
			camObjFound = None
			exitcam = 0
				
			for obj in proj:
				if obj.isValid():
					if found: # found 2nd one, exit cam
						playerExitCam(pid,projTemp,pteam,proj)
						exitcam = 1
					found = 1
					camObjFound = obj

			if exitcam or not found:
				continue

			cur = None
			if not pid in gCams[pteam]:
				cur = playerEnterCam(pid,camObjFound,pteam,proj)
			else:
				cur = gCams[pteam][pid]

			if cur is None: #shouldn't happen
				if g_debug:
					print 'ERR: cur = gCams[pteam][pid] == None'
				continue

			# if we already have a valid trackable then we're done
			if cur.trackingProj:
				if cur.trackingProj.isValid():
					try:
						trackProjPos = cur.trackingProj.getPosition()
						if trackProjPos != ZERO_VECT3:
							continue
					except:
						if g_debug:
							print 'ERR: trackProjPos = cur.trackingProj.getPosition()'



			mainProjTemplate = CAM_TEMPLATES[camObjFound.templateName]
			foundGoodProj = 0
			for mnPrj in allProjs:
				if mnPrj.templateName == mainProjTemplate:
					if mnPrj.isValid():
						try:
							newTrackPos = mnPrj.getPosition()
							if newTrackPos != ZERO_VECT3:
								foundGoodProj = 1
								#mnPrjGood = mnPrj
								break
						except:
							if g_debug:
								print 'ERR: newTrackPos = mnPrj.getPosition()'


			if foundGoodProj:
				cur.trackingProj = mnPrj # mnPrjGood
				cur.holdLastTrackSpotCount = HOLD_LAST_TRACKING_POSITION_COUNT
				gTrackingActive = 1




# #################################################	
# #################### Helper / untils ####################
# #################################################	


def killProjectilesOfTemplate(temp,projList):
	#print '-=-=-killProjectilesOfTemplate-=-=-=:: ' + str(temp) + str(projList)
	if projList is None:
		#print 'MAKING LIST'
		projList = []
		allProjs = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.GenericProjectile')
		#print allProjs
		if len(allProjs) < 1:
			return 
		for prj in allProjs:
			if prj.templateName == temp:
				projList.append(prj)
				
	if len(projList) < 1:
		return 
	for prj in projList:
		try:
			#print 'KILLING a' + str(prj.templateName)
			prj.setDamage(-1)
		except:
			if g_debug: 
				print 'BAD killProjectilesOfTemplate: ' + str(temp)
				print 'isvalid: ' + str(prj.isValid())



def setFirearmProjectile(firearmTemp, projTemp):
	host.rcon_invoke("ObjectTemplate.active " + firearmTemp)
	host.rcon_invoke("ObjectTemplate.projectileTemplate " + projTemp)	
	host.rcon_invoke("ObjectTemplate.setNetworkableInfo BasicInfo")


def setEntryPoint(entryTemp, radius):
	if g_debug:
		print 'setEntryPoint: ' + str(entryTemp) + ' ' + str(radius)
	host.rcon_invoke("ObjectTemplate.active " + entryTemp)
	host.rcon_invoke("ObjectTemplate.setEntryRadius " + radius)
	host.rcon_invoke("ObjectTemplate.setNetworkableInfo BasicInfo")


def makeVect(p1,p2):
	return (p2[0] - p1[0],p2[1] - p1[1],p2[2] - p1[2])


def vectMag(v):
	return math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2])


def killArtyTimer():
	#print '==== killArtyTimer ====='
	global gCamTimer
	if gCamTimer:
		try: gCamTimer.destroy()
		except: pass
	gCamTimer = None


def checkArtyTimer():
	global gCamTimer
	#global gSpots
	totalSpotters = len(gSpots[1]) + len(gSpots[2])
	totalArties = len(gPlayersInArty[1]) + len(gPlayersInArty[2])


	if totalArties > 0 and totalSpotters > 0:
		if gCamTimer is None:
			gCamTimer = bf2.Timer(camTimerHandler, NORMAL_TIMER_PERIOD, 1)
			gCamTimer.setRecurring(NORMAL_TIMER_PERIOD)
	else: #make sure its shutoff
		killArtyTimer()


def	checkArtyCamsEnabled(team):
	spotsForTeam = len(gSpots[team])
	faproj = ''
	for piaid in gPlayersInArty[team].keys():
		vehTemp = gPlayersInArty[team][piaid]
		fireArm = NULL_CAMS[vehTemp][0]

		if spotsForTeam > 0: # activate
			faproj = ARTY_VEHICLES[vehTemp]
		else: # disable
			faproj = NULL_CAMS[vehTemp][1]

		setFirearmProjectile(fireArm, faproj)



# ########################################################
# ################### BF2 EVENT HANDLERS #######################
# ########################################################	


def onEnterVehicleARTY(player, vehicle, freeSoldier = False):
	global gCamTimer
	global gPlayersInArty
	vehTemp = vehicle.templateName
	if g_debug:
		print 'onEnterVehicleARTY: ' + str(vehTemp)

	if vehTemp in ARTY_VEHICLES:

		pid = player.index
		team = player.getTeam()

		gPlayersInArty[team][pid] = (vehTemp)

		spotsForTeam = len(gSpots[team])
		fireArm = NULL_CAMS[vehTemp][0]

		if spotsForTeam > 0: # activate
			faproj = ARTY_VEHICLES[vehTemp]
		else: # disable
			faproj = NULL_CAMS[vehTemp][1]

		setFirearmProjectile(fireArm, faproj)
		checkArtyTimer()

def onExitVehicleARTY(player, vehicle):
	if g_debug:
		print 'onExitVehicleARTY: ' + str(vehicle.templateName)

	vehTemp = vehicle.templateName

	if not vehTemp in ARTY_VEHICLES:
		return

	team = player.getTeam()
	pid = player.index
	if not pid in gPlayersInArty[team]:
		return

	pia = gPlayersInArty[team][pid]
	del gPlayersInArty[team][pid]


	ptype = ARTY_VEHICLES[pia]
	playerExitCam(pid,ptype,team,None)
	checkArtyTimer()#shutdown timer?



def onRemoteCommandARTY(playerId, cmd):
	if cmd == 'dcon_up_arty':
		if playerId == -1:#local testing
			playerNextSpotter(255)
		else:
			playerNextSpotter(playerId)
		return
		
	if cmd == 'dcon_arty_set_spot':
		if playerId == -1: #local testing
			playerId = 255
		plyrObj = bf2.playerManager.getPlayerByIndex(playerId)
		pt = plyrObj.getTeam()
		v = plyrObj.getDefaultVehicle()
		pos = v.getPosition()
		rot = v.getRotation()
		pid = plyrObj.index
		pName = plyrObj.getName()
		newSpot(pos,rot,pid,pt,pName)


#  73E test spots
#def onChatMessageARTY(playerId, text, channel, flags):	
		
		
	# if text == 'all':
		# newSpot((53.026, 56.70, -154.03), (65.9, 0.0, 0.0), 3, 2,"dummy spotter id=3")
		# newSpot((-306.87, 56.30, 223.65), (31.37, 0.0, 0.0), 4, 2,"dummy spotter id=4")
		# newSpot((304.68, 56.73, 328.37), (0.0, 0.0, 0.0), 5, 2,"dummy spotter id=5")
			
	# if text == 's1':
		# newSpot((53.026, 56.70, -154.03), (65.9, 0.0, 0.0), 3, 2,"dummy spotter id=3")

	# if text == 's2':
		# newSpot((-306.87, 56.30, 223.65), (31.37, 0.0, 0.0), 4, 2,"dummy spotter id=4")

	# if text == 's3':
		# newSpot((304.68, 56.73, 328.37), (0.0, 0.0, 0.0), 5, 2,"dummy spotter id=5")
