#********************************************************************************************************
# Name: Rally Racing for Battlefield 2
# Author: Mach10 <mmccormack@MNMCreationsllc.com>
# Version 0.1 Beta
# Copyright  MNM Creations, LLC  - www.mnmcreationsllc.com/games
#----------------------------------------------------------------------------------------------------------------------------
# Conditions and Limitations: You may use this file locally, on listening and dedicated servers. You MAY NOT 
# copy, redistributed, post, modify this file with out the expressed permission from the author/copyright holder.
# You may contact me and request these rights via e-mail: mach10@comcast.net
#This software is provided "as is". In no event shall the author, authors or contributors be held
#liable for  any direct, indirect, incidental, special, exemplary, or consequential damages (including)
#but not limited to, procurement of substitute good or services; loss of use, data or profits; or 
#business interruption) however caused and on any theory of liability, whether in contract, strict liability
#arising in any way out of the use of this software, even if advised of the possibility of such damage.
#=====
# Purpose: The file creates a new game mode for Battlefield 2, Rally Racing.
#=====
# Game Mode Information: You must spawn on the Coalition team, and take a DPV from your non-cap to the other
# team's non-cap and back again. The player who completes 3 full laps will win.
#*********************************************************************************************************

Top = 0
Middle = 1
Bottom = 2

# Edit This To Increase The Number Of Laps Per Map! 4 = 1 lap, 8 = 2 laps, 12 = 3 laps
number_of_laps = 4

firstPlace = None
secondPlace = None
thirdPlace = None
displayWinnersTimerIndex = 0
numberOfPlayers = 0


import host
import bf2
import math
from game.scoringCommon import addScore, RPL
from bf2 import g_debug

g_controlPoints = [] # cache, as this map won't change


def init():
	# events hook
	host.registerGameStatusHandler(onGameStatusChanged)
	if host.sgl_getIsAIGame() == 1:
		host.sh_setEnableCommander(0)
	else:
		host.sh_setEnableCommander(0)
		
	if g_debug: print "gpm_rally.py initialized"
		
def deinit():
	bf2.triggerManager.destroyAllTriggers()
	global g_controlPoints
	g_controlPoints = []
	host.unregisterGameStatusHandler(onGameStatusChanged)
	if g_debug: print "gpm_rally.py uninitialized"

def onGameStatusChanged(status):

	global g_controlPoints
	if status == bf2.GameStatus.Playing: 
		g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
		for obj in g_controlPoints:
			radius = float(obj.getTemplateProperty('radius'))
			checkpointnum = int(obj.getTemplateProperty('ControlPointID'))
			if checkpointnum == 1:
				id = bf2.triggerManager.createRadiusTrigger(obj, FinishTrigger, '<<PCO>>', radius, (1, 2, 3))
			elif checkpointnum == 2:
				id = bf2.triggerManager.createRadiusTrigger(obj, CheckPointTrigger1, '<<PCO>>', radius, (1, 2, 3))
			elif checkpointnum == 3:
				id = bf2.triggerManager.createRadiusTrigger(obj, CheckPointTrigger2, '<<PCO>>', radius, (1, 2, 3))
			elif checkpointnum == 4:
				id = bf2.triggerManager.createRadiusTrigger(obj, CheckPointTrigger3, '<<PCO>>', radius, (1, 2, 3))


			obj.triggerId = id
			obj.lastAttackingTeam = 0
		
		bf2.gameLogic.setTickets(1, 1)
		bf2.gameLogic.setTickets(2, 1)
		
		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)

		host.registerHandler('PlayerSpawn', onPlayerSpawn)
		host.registerHandler('EnterVehicle', onEnterVehicle)
		host.registerHandler('ExitVehicle', onExitVehicle)
		
		endOfRoundTimer = bf2.Timer(endOfRound, 0 , 1)
		endOfRoundTimer.setRecurring(5.0)
		
		if g_debug: print "Rally gamemode initialized."
		print 'Author is Mach10, mach10@comcast.net'

	else:
		bf2.triggerManager.destroyAllTriggers()
		g_controlPoints = []
	
DOWNWARDS = -1
UPWARDS = 1	

def displayPlayerPOS(data):
	for p in bf2.playerManager.getPlayers():
		if (p.getName() == "gunzip") and p.isAlive():
			vehicle = p.getVehicle()
			pPos = vehicle.getPosition()
			print pPos


def CheckPointTrigger1(triggerId, cp, vehicle, enter, userData):

	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue 
		if o.getParent(): continue
		occupyingPlayers = o.getOccupyingPlayers()
		for p in occupyingPlayers:
			if p != occupyingPlayers[0]: 
				continue
				
			if p.isAlive() and not p.isManDown():
				if not p.killed:
					if p.getDefaultVehicle() != p.getVehicle():
						if p.getTeam() == 2 and (p.score.cpCaptures % 4) == 0:
							host.rcon_invoke("game.sayAll \"" + "Player " + p.getName() + " has passed Check Point 1"  + "\"")
							p.score.cpCaptures += 1
							addScore(p, 1, RPL)
			
def CheckPointTrigger2(triggerId, cp, vehicle, enter, userData):		

	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue 
		if o.getParent(): continue
		occupyingPlayers = o.getOccupyingPlayers()
		for p in occupyingPlayers:
			if p != occupyingPlayers[0]: 
				continue
				
			if p.isAlive() and not p.isManDown():
				if not p.killed:
					if p.getDefaultVehicle() != p.getVehicle():
						if p.getTeam() == 2 and (p.score.cpCaptures % 4) == 1:
							host.rcon_invoke("game.sayAll \"" + "Player " + p.getName() + " has passed Check Point 2"  + "\"")
							p.score.cpCaptures += 1
							addScore(p, 1, RPL)
			
def CheckPointTrigger3(triggerId, cp, vehicle, enter, userData):		

	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue 
		if o.getParent(): continue
		occupyingPlayers = o.getOccupyingPlayers()
		for p in occupyingPlayers:
			if p != occupyingPlayers[0]: 
				continue
				
			if p.isAlive() and not p.isManDown():
				if not p.killed:
					if p.getDefaultVehicle() != p.getVehicle():
						if p.getTeam() == 2 and (p.score.cpCaptures % 4) == 2:
							host.rcon_invoke("game.sayAll \"" + "Player " + p.getName() + " has passed Check Point 3"  + "\"")
							p.score.cpCaptures += 1
							addScore(p, 1, RPL)
			


def FinishTrigger(triggerId, cp, vehicle, enter, userData):				
	global firstPlace
	global secondPlace
	global thirdPlace
	global displayWinnersTimerIndex
	global number_of_laps
	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue
		if o.getParent(): continue
		occupyingPlayers = o.getOccupyingPlayers()
		for p in occupyingPlayers:
			if p != occupyingPlayers[0]: 
				continue
				
			if p.isAlive() and not p.isManDown():
				if not p.killed:
					if p.getDefaultVehicle() != p.getVehicle():
						if p.getTeam() == 2 and (p.score.cpCaptures % 4) == 3 and p.score.cpCaptures != 0:
							host.rcon_invoke("game.sayAll \"" + "Player " + p.getName() + " has Completed a Lap"  + "\"")
							p.score.cpCaptures += 1
							addScore(p, 1, RPL)
							if p.score.cpCaptures == number_of_laps: 
								if checkWinners(p) == 1:
									displayWinnersTimer = bf2.Timer(displayWinners, 0, 1)
									displayWinnersTimer.setRecurring(3.0)
							
						else:
							print 'Error: There can not be two teams in racing!'

def endOfRound(data):
	global displayWinnersTimerIndex
	if displayWinnersTimerIndex == 5:
		host.rcon_invoke("game.sayAll \"" + "Round will restart!"  "\"")
		displayWinnersTimerIndex += 1
	elif displayWinnersTimerIndex == 6:
		displayWinnersTimerIndex += 1
	elif displayWinnersTimerIndex == 7:
		host.rcon_invoke("game.sayAll \"" + "Round will restart!"  "\"")
		displayWinnersTimerIndex += 1
	elif displayWinnersTimerIndex == 8:
		CleanUp()

def displayWinners(data):
	global firstPlace
	global secondPlace
	global thirdPlace
	global displayWinnersTimerIndex
	if not secondPlace:
		secondPlace = 'Player Missing'
	if not thirdPlace:
		thirdPlace = 'Player Missing'
	host.rcon_invoke("game.sayAll \"" + "Round Results:" + "\"")
	host.rcon_invoke("game.sayAll \"" + "First Place " + firstPlace + "\"")
	host.rcon_invoke("game.sayAll \"" + "Second Place " + secondPlace + "\"")
	host.rcon_invoke("game.sayAll \"" + "Third Place " + thirdPlace + "\"")
	host.rcon_invoke("game.sayAll \"" + "==================================" + "\"")
	displayWinnersTimerIndex += 1
	if displayWinnersTimerIndex == 5:
		displayWinnersTimer.destroy()
		displayWinnersTimer = None
	
def CleanUp():
	global firstPlace
	global secondPlace
	global thirdPlace
	global displayWinnersTimerIndex
	global numberOfPlayers
	global numberOfPlayersChecked 
	firstPlace = None
	secondPlace = None
	thirdPlace = None
	displayWinnersTimerIndex = 0
	numberOfPlayers = 0
	numberOfPlayersChecked = 0
	host.sgl_endGame(2, 3)
	
def checkWinners(player):
	numberOfPlayers = host.pmgr_getNumberOfPlayers()
	
	if numberOfPlayers == 1:
		if onePlayerRally(player) == 1:
			return 1
		
	elif numberOfPlayers == 2:
		if twoPlayerRally(player) == 1:
			return 1
		
	elif numberOfPlayers >= 3:
		if threePlayerRally(player) == 1:
			return 1
		
def onePlayerRally(player):
	global firstPlace
	
	if not firstPlace:
		firstPlace = player.getName()
		return 1

def twoPlayerRally(player):
	global firstPlace
	global secondPlace
	thisName = player.getName()
	
	if not firstPlace:
		firstPlace = thisName

	elif firstPlace is not thisName:
		secondPlace = thisName
		return 1

def threePlayerRally(player):
	global firstPlace
	global secondPlace
	global thirdPlace
	thisName = player.getName()

	if not firstPlace:
		firstPlace = thisName

	elif not secondPlace and firstPlace is not thisName:
		secondPlace = thisName
		
	elif not thirdPlace and firstPlace is not thisName and secondPlace is not thisName:
		thirdPlace = thisName
		return 1


def onPlayerSpawn(player, soldier):
	player.killed = False

	
def onEnterVehicle(player, vehicle, freeSoldier = False):
	updateCaptureStatus(vehicle)

def onExitVehicle(player, vehicle):
	print "Player exiting ", player.getName()
	cp = getOccupyingCP(player)
	player.setIsInsideCP(cp != None and cp.cp_getParam('unableToChangeTeam') == 0)
	updateCaptureStatus(vehicle)
		

def updateCaptureStatus(vehicle):	

	rootVehicle = bf2.objectManager.getRootParent(vehicle)
	playersInVehicle = rootVehicle.getOccupyingPlayers()
	for p in playersInVehicle:
		print "Players in vehicle ", p.getName()
		cp = getOccupyingCP(p)
		p.setIsInsideCP(cp != None and cp.cp_getParam('unableToChangeTeam') == 0 and p == playersInVehicle[0])
	
def getOccupyingCP(player):
	vehicle = player.getVehicle()
	playerPos = vehicle.getPosition()
	
	closestCP = None
	if len(g_controlPoints) == 0: return None
	for obj in g_controlPoints:
		distanceTo = getVectorDistance(playerPos, obj.getPosition())
		if closestCP == None or distanceTo < closestCPdist:
			closestCP = obj
			closestCPdist = distanceTo
	
	pcos = bf2.triggerManager.getObjects(closestCP.triggerId)
	for o in pcos:
		if o == player.getDefaultVehicle():
			return closestCP
		else:
			for p in o.getOccupyingPlayers():
				if p == player:
					return closestCP
	
	return None

def getVectorDistance(pos1, pos2):
	diffVec = [0.0, 0.0, 0.0]
	diffVec[0] = math.fabs(pos1[0] - pos2[0])
	diffVec[1] = math.fabs(pos1[1] - pos2[1])
	diffVec[2] = math.fabs(pos1[2] - pos2[2])
	 
	return math.sqrt(diffVec[0] * diffVec[0] + diffVec[1] * diffVec[1] + diffVec[2] * diffVec[2])
	
