# Rally point system for 21CW
#
# Written/Adapted by [R-DEV]e-Gor
# Based on the Project Reality rally point system written by [R-DEV]dizzy and [R-DEV]dbzao
#

import host
import bf2

from game.spawners import createSpawner

# config #
minDistanceFromCP = 65
minNumInSquad = 3
squadMembersDistance = 15
minNumSquadClose = 3
rallyInterval = 90

# # # # # #

distanceFromSpawnPoint = 10
g_bases = {}
g_controlPoints = []
g_rps = { 1: {}, 2: {} }

Top = 0
Middle = 1
Bottom = 2

def init():
	host.registerGameStatusHandler(onGameStatusChanged)
	host.registerHandler('RemoteCommand', onRemoteCommand)
	host.registerHandler('PlayerSpawn', onPlayerSpawn)
	host.registerHandler('PlayerChangedSquad', onPlayerChangedSquad)
	

def onGameStatusChanged(status):
	global g_bases, g_controlPoints
	
	if status != bf2.GameStatus.Playing:
		return
	
	# Get all control points
	g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
	
	# Set base flags
	g_bases = {
		1: [],
		2: []
	}
	
	# Clean the RPs
	g_rps = { 1: {}, 2: {} }
	
	last = -999
	first = 999
	bases = 0
	for obj in g_controlPoints:
		
		obj.lastAttackingTeam = 0
		if obj.cp_getParam('team') > 0:
			obj.flagPosition = Top
		else:
			obj.flagPosition = Bottom
		
		try:
			sgid = int(obj.cp_getParam('supplyGroupId'))
			if sgid == -1:
				continue
			bases += 1
		except: continue
		
		if sgid < first:
			first = sgid
		if sgid > last:
			last = sgid
	
	if bases >= 2:
		for obj in g_controlPoints:
			try:
				sgid = int(obj.cp_getParam('supplyGroupId'))
			except: continue
			
			if sgid == first:
				g_bases[1].append( obj )
			if sgid == last:
				g_bases[2].append( obj )
	else:
		g_bases[1] = g_controlPoints
		g_bases[2] = g_controlPoints
	
def onRemoteCommand(playerId, cmd):
	global g_rps
	
	if cmd != 'rally':
		return
	
	if playerId == -1:
		playerId = 255 # single player or lan host
	
	player = bf2.playerManager.getPlayerByIndex(playerId)
	if not canSpawnRally(player):
		return
	
	squadId = player.getSquadId()
	team = player.getTeam()
	teamName = bf2.gameLogic.getTeamName(team)
	position = player.getDefaultVehicle().getPosition()
	
	templateName = 'rallypoint_' + teamName + '_' + str(squadId)
	
	deleteRallyPoints(templateName)
	createSpawner( templateName + '_spawner', (position[0], position[1]-1.0, position[2]), None, True, {'team': team } )
	
	g_rps[team][squadId] = now()


def deleteRallyPoints(templateName):
	objs = host.rcon_invoke('object.listObjectsOfTemplate ' + templateName)
	objs = objs.split()
	for i in range( (len(objs)/10)  ):
		id = (i * 10) + 3
		host.rcon_invoke('object.delete id' + str(objs[id]) )


def onPlayerChangedSquad(player, oldSquad, newSquad):
	
	# check only if he left a squad
	if newSquad != 0:
		return
	
	team = player.getTeam()
	teamName = bf2.gameLogic.getTeamName(team).lower()
	
	if len( getPlayersOfSquad( team, oldSquad ) ) < minNumInSquad:
		deleteRallyPoints( 'rallypoint_' + teamName + '_' + str(oldSquad) )
	

def canSpawnRally(player):
	
	playerPos = player.getDefaultVehicle().getPosition()
	playerTeam = player.getTeam()
	playerName = player.getName()
	playerSquad = player.getSquadId()
	
	# check if SL (otherwise anyone can run rcon command from the console)
	if not player.isSquadLeader():
		return False
	
	# check whether the player is in a vehicle
	if player.getVehicle().templateName != player.getDefaultVehicle().templateName:
		sendTeamMessage(playerTeam, playerName + ', you cannot place a rally point when in a vehicle.')
		return False
	
	# check the last time the rally was spawned
	try:
		if now() - g_rps[playerTeam][playerSquad] <= rallyInterval:
			sendTeamMessage(playerTeam, playerName + ', you cannot place a rally point so soon after your last.')
			return False
	except: pass
	
	# check distance from CPs
	for cp in g_controlPoints:
		if getSquareHorizDistance( playerPos, cp.getPosition() ) < minDistanceFromCP ** 2:
			sendTeamMessage(playerTeam, playerName + ', you are too close to a CP to set a rally point.')
			return False
	
	# get the list of squad members
	squadMembers = getPlayersInSquad(player)
	
	# check enough players in squad
	if len( squadMembers ) < minNumInSquad:
		sendTeamMessage(playerTeam, playerName + ', you must have at least ' + str(minNumInSquad) + ' players in your squad to place a rally point.')
		return False
	
	# check number of squad members nearby
	numSquadClose = 0
	for p in squadMembers:
		if p.isAlive() and not p.isManDown() and getSquareHorizDistance(playerPos, p.getDefaultVehicle().getPosition() ) < squadMembersDistance ** 2:
			numSquadClose += 1
	if numSquadClose < minNumSquadClose:
		sendTeamMessage(playerTeam, playerName + ', you need at least ' + str(minNumSquadClose) + ' squad members nearby to place a rally point.')
		return False
	
	return True


def sendTeamMessage(team, msg):
	host.rcon_invoke('game.sayTeam ' + str(team) + ' "' + msg + '"')

	
def getPlayersInSquad(player):
	return getPlayersOfSquad( player.getTeam(), player.getSquadId() )

def getPlayersOfSquad(team,squad):
	
	players = []
	if team not in [1,2] or squad == 0:
		return players
		
	for p in bf2.playerManager.getPlayers():
		if p.getTeam() == team and p.getSquadId() == squad:
			players.append(p)
	
	return players


def getSquadLeader(player):
	team = player.getTeam()
	squad = player.getSquadId()
	
	if squad == 0:
		return player
	
	for p in bf2.playerManager.getPlayers():
		if p.getTeam() == team and p.getSquadId() == squad and p.isSquadLeader():
			return p


def onPlayerSpawn(player, soldier):
	player.rallyPointSpawnCheckTimer = bf2.Timer(onRallyPointSpawnCheckTimer, 0.5, 1, player)


# This function makes sure that no one spawns on another team's rally point - if they try they'll
# be teleported to their own or their main base if their squad does not have a rally point

# Not tested the base warping yet, but it *should* work.

def onRallyPointSpawnCheckTimer(player):
	try:
		if player.rallyPointSpawnCheckTimer:
			player.rallyPointSpawnCheckTimer.destroy()
	except: pass
	player.rallyPointSpawnCheckTimer = None

	# Commander can spawn in any rally point
	if player.isCommander():
		return
	
	# Either spawned on SL in a vehicle or spawned on APC
	if player.getDefaultVehicle().templateName != player.getVehicle().templateName:
		print "Spawned in vehicle"
		return

	playerPos = player.getDefaultVehicle().getPosition()
	playerTeam = player.getTeam()
	playerTeamName = bf2.gameLogic.getTeamName(playerTeam).lower()
	playerSquadId = player.getSquadId()
	distanceFromSpawnPointSquared = distanceFromSpawnPoint * distanceFromSpawnPoint
	
	# Check if they are within X meters of their SL
	if playerSquadId != 0 and not player.isSquadLeader():
		squadLeader = getSquadLeader(player)
		if squadLeader.isAlive() and not squadLeader.isManDown():
			if getSquareHorizDistance(squadLeader.getDefaultVehicle().getPosition(), playerPos) < distanceFromSpawnPointSquared:
				print "Spawned on SL"
				return
	
	# Correct Player Rally Point
	correctTemplateName = 'rallypoint_' + playerTeamName + '_' + str(playerSquadId)
	wrongRally = False
	
	# Check all rally points
	for i in range(10):
		for rallyPoint in bf2.objectManager.getObjectsOfTemplate('rallypoint_' + playerTeamName + '_' + str(i)):
			try:
				if not rallyPoint.isValid() or rallyPoint.getPosition() == (0,0,0):
					continue
			except: continue
			
			# Check to see if it's a valid rally point
			if getSquareHorizDistance(rallyPoint.getPosition(), playerPos) < distanceFromSpawnPointSquared:
				
				templateName = rallyPoint.templateName.lower()
				if correctTemplateName == templateName or 'rallypoint_' + playerTeamName + '_0' == templateName:
					print "Spawned at correct rally point"
					return
				else:
					print "Spawned at incorrect rally point"
					wrongRally = True

	if not wrongRally:
		return

	# Not a good spawn, warp the player to correct RP or HOME
	if playerSquadId != 0:
		
		for rallyPoint in bf2.objectManager.getObjectsOfTemplate(correctTemplateName):
			try:
				if not rallyPoint.isValid() or rallyPoint.getPosition() == (0,0,0):
					continue
			except: continue
			
			print "Spawned at wrong RP - warped to correct RP"
			player.getDefaultVehicle().setPosition( rallyPoint.getPosition() )
			return

	print "Spawned at wrong RP - warped to main base"
	player.getDefaultVehicle().setPosition( g_bases[playerTeam][0].getPosition() ) 


def getSquareHorizDistance(pos1, pos2):

	dx = pos1[0] - pos2[0]
	dz = pos1[2] - pos2[2]
	return ( dx * dx ) + ( dz * dz )


def now():
	return int( host.timer_getWallTime() )

