# Spawners creation and deletion
# 
# Coded by [R-DEV]dbzao
# 

import host
import bf2

g_spawners = {}

spawnerDefault = {
	'minSpawnDelay': 99999,
	'maxSpawnDelay': 99999,
	'maxNrObjectSpawned': 1,
	'distance': 99999,
	'timeToLive': 99999,
	'damageWhenLost': 0,
	'teamOnVehicle': 1,
	'team': 1
}

def init():
	host.registerGameStatusHandler(onGameStatusChanged)
	
def onGameStatusChanged(status):
	global g_spawners
	
	if status == bf2.GameStatus.EndGame:

		deletion = []
		for name,index in g_spawners.iteritems():
			deletion.append( name )

		for name in deletion:
			deleteSpawner( name )
		
		g_spawners = {}

def createSpawner( name, position=None, rotation=None, delete=True, properties={} ):
	global g_spawners
	
	# set the default template if not passed
	defaults = {}
	if 'setObjectTemplate 1' not in properties and 'setObjectTemplate 2' not in properties:
		template = name.replace('_spawner','')
		defaults['setObjectTemplate 1'] = template
		defaults['setObjectTemplate 2'] = template
	
	# set the default properties
	for p,v in spawnerDefault.iteritems():
		defaults[p] = v
		
	# update the default properties with the custom ones
	for p,v in properties.iteritems():
		defaults[p] = v
	
	# can't have spawners with the same name
	if spawnerExists( name ):
		return False
		
	# create the spawner template
	host.rcon_invoke('ObjectTemplate.create objectSpawner ' + name )
	host.rcon_invoke('ObjectTemplate.activeSafe objectSpawner ' + name )
	
	# set the properties
	for p,v in defaults.iteritems():
		if str( v ) != '': 
			host.rcon_invoke('ObjectTemplate.' + p + ' ' + str( v ) )
	
	# create the spawner
	index = host.rcon_invoke('Object.create ' + name ).replace('\n','')
	
	# creation failed
	if index == 'Unauthorised method!':
		return False
	
	# set the spawner properties
	if position:
		host.rcon_invoke('Object.absolutePosition %s/%s/%s' % position )
	if rotation:
		host.rcon_invoke('Object.rotation %s/%s/%s' % rotation )
	
	# add the spawner to the list of created by code
	g_spawners[name] = index
	
	# delete the spawner if requested
	if delete:
		deleteSpawner( name )

	return True

def spawnerExists( name ):
	if name in g_spawners:
		return True
	return False

def deleteSpawner( name ):
	global g_spawners
	
	if not spawnerExists( name ):
		return False
	
	# execute the deletion
	host.rcon_invoke('Object.delete ' + g_spawners[name] )
	
	# delete from the list
	try:
		del g_spawners[name]
	except: pass
	
	return True
	
