# Losing a vehicle costs tickets for OPT
#
# Written/Adapted by HWFlo
# Based on ThreatAndWorth written by SHAnders
#
# Version 1.0 - initial version
#
# Version 1.1 - added logging etc.
#
# Version 1.2 - improved scoring based on the vehicle used for attacking

import host
import bf2
# from bf2.stats.constants import *
import constants_21cw
from constants_21cw import *
from bf2 import g_debug
from bf2.stats.stats import onGameStatusChanged

from game.utils.debugging import *

global msgtimer
global debug

debug = False

# ------------------------------------------------------------------------
# Constants
# ------------------------------------------------------------------------

WORTHSCHEMA = { VEHICLE_TYPE_ARMOR : [ 6 ],
		VEHICLE_TYPE_AVIATOR : [ 8 ],
		VEHICLE_TYPE_AIRDEFENSE	: [ 1 ],
		VEHICLE_TYPE_HELICOPTER	: [ 6 ],
		VEHICLE_TYPE_TRANSPORT	: [ 3 ],
		VEHICLE_TYPE_ARTILLERY	: [ 1 ],
		VEHICLE_TYPE_GRNDDEFENSE : [ 1 ],
		VEHICLE_TYPE_PARACHUTE : [ 1 ],
		VEHICLE_TYPE_SOLDIER : [ 0 ] ,
		VEHICLE_TYPE_NIGHTVISION : [ 0 ] ,
		VEHICLE_TYPE_GASMASK : [ 0 ] ,
		VEHICLE_TYPE_UNKNOWN : [ 0 ] ,
		}

WORTHSCHEMA2 = { VEHICLE_TYPE_ARMOR : [ 5 ],
		VEHICLE_TYPE_AVIATOR : [ 10 ],
		VEHICLE_TYPE_AIRDEFENSE	: [ 4 ],
		VEHICLE_TYPE_HELICOPTER	: [ 8 ],
		VEHICLE_TYPE_TRANSPORT	: [ 1 ],
		VEHICLE_TYPE_ARTILLERY	: [ 4 ],
		VEHICLE_TYPE_GRNDDEFENSE : [ 1 ],
		VEHICLE_TYPE_PARACHUTE : [ 1 ],
		VEHICLE_TYPE_SOLDIER : [ 0 ] ,
		VEHICLE_TYPE_NIGHTVISION : [ 0 ] ,
		VEHICLE_TYPE_GASMASK : [ 0 ] ,
		VEHICLE_TYPE_UNKNOWN : [ 0 ] ,
		}

DEFAULT_VICTIM_WORTH = 0      # Victim will allways be worth this + whats in the WORTHSCHEMA
PUBLIC_SCORES = 2             # Score earnings will be made public to the following: [0 = None, 1 = Team, 2 = All] (for debugging only)

WELCOME_MSG = "Server is using VehicleCostsTickets script v1.2"

# ------------------------------------------------------------------------
# Variables
# ------------------------------------------------------------------------

MsgTimer = None # Welcome msg Timer
DestName = ""

# ------------------------------------------------------------------------
# Init
# ------------------------------------------------------------------------
def init():
   print "ThreatAndWorth init 1"
   # only active when TKPunish is enabled (can be removed once fully tested)
   #if bf2.serverSettings.getTKPunishEnabled():
   host.unregisterGameStatusHandler(onGameStatusChanged) # We need to unregister this to change endscores
   host.registerGameStatusHandler(onGameStatusChangedThreatAndWorth)
   host.registerGameStatusHandler(onGameStatusChanged) # We need to register it again after our own handler to make the game work :)
   msgtimer = bf2.Timer(onMsgTimer,10,1)
   if bf2.serverSettings.getTKPunishEnabled():
	   debug = True
   print "ThreatAndWorth init 2"
   #print "ThreatAndWorth init 3"
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# onGameStatusChangedThreatAndWorth
# ------------------------------------------------------------------------
def onGameStatusChangedThreatAndWorth(status):
   if status == bf2.GameStatus.Playing:
      host.registerHandler('PlayerKilled', onPlayerKilled)
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
# onPlayerKilled
# ------------------------------------------------------------------------
def onPlayerKilled(victim, attacker, weapon, assists, object):
   victimVehicle = victim.getVehicle()

   #for debugging purposes
   if debug: sendMsgAll("DEBUG: I am here")

   # killed by remote controlled vehicle, no score awarded in this game
   if victimVehicle and victimVehicle.getIsRemoteControlled():
      pass

   # no attacker, killed by object
   elif attacker == None:
      pass

   # killed by self
   elif attacker == victim:
      pass

   # killed by own team
   elif attacker.getTeam() == victim.getTeam():
      pass

   # killed by enemy
   else:
      CalculateThreatAndWorth(attacker, victim)
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# processBonus
# ------------------------------------------------------------------------
def processBonus(attacker, victim, points):
   # commander doesnt get bonus for regular actions, only for pure commander tasks.
   if not attacker.isCommander():
      attacker.score.score += points

   # team tickets
   tickets = bf2.gameLogic.getTickets(victim.getTeam())
   if debug: sendMsgAll("DEBUG: tickets old " + str(tickets))
   tickets -= points
   if debug: sendMsgAll("DEBUG: tickets new " + str(tickets))
   bf2.gameLogic.setTickets(victim.getTeam(), tickets)
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# Callback function for timer, that delays a message at round start:
# ------------------------------------------------------------------------
def onMsgTimer(data):
   try:
	   if msgtimer:
		   msgtimer.destroy()
   except: pass
   msgtimer = None
   sendMsgAll( WELCOME_MSG )
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
# Send message to all on server
# ------------------------------------------------------------------------
def sendMsgAll(msg):
   host.rcon_invoke("game.sayAll \"" + str(msg) + "\"")
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# Send message to all on a team
# ------------------------------------------------------------------------
def sendMsgTeam(msg, team):
   host.rcon_invoke("game.sayTeam " + str(team) + " \"" + str(msg) + "\"")
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# Calculate threat and worth points
# ------------------------------------------------------------------------
def CalculateThreatAndWorth(attacker, victim):

   points = GetPlayerWorth( victim ) #+ GetAdditionalVictimWorth(attacker, victim)

   if points > 0:
      processBonus(attacker, victim, points)
      if PUBLIC_SCORES == 2:
         if debug: sendMsgAll( attacker.getName() + " earn points: " + str(points))
      elif PUBLIC_SCORES == 1:
         if debug: sendMsgTeam( attacker.getName() + " earn points: " + str(points), attacker.getTeam())
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# Calculate additional victim worth
# ------------------------------------------------------------------------
def GetAdditionalVictimWorth(attacker, victim):
   #this function is actually not used!
   attackerTeamMsg = ""
   victimTeamMsg = ""

   worth = DEFAULT_VICTIM_WORTH

   # SQUADLEADER
   if victim.isSquadLeader():
      worth += VICTIM_SQUADLEADER
      attackerTeamMsg = attacker.getName() + " has eliminated a enemy squadleader " + victim.getName()

   # COMMANDER
   if victim.isCommander():
      worth += VICTIM_COMMANDER
      attackerTeamMsg = attacker.getName() + " has eliminated enemy commander " + victim.getName()
      victimTeamMsg = "Your commander has been eliminated"

   if attackerTeamMsg != "":
      sendMsgTeam(attackerTeamMsg, attacker.getTeam())

   if victimTeamMsg != "":
      sendMsgTeam(victimTeamMsg, victim.getTeam())

   return worth
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# Calculate a players worth
# ------------------------------------------------------------------------
def GetPlayerWorth( player ):
   vehicle = player.getVehicle()
   rootVehicle = getRootParent(vehicle)
   vehicleType = getVehicleType(rootVehicle.templateName)

   #for debugging purposes
   if debug: sendMsgAll("DEBUG: Destroyed vehicle " + rootVehicle.templateName + " - type: " + str(vehicleType))

   DestName = vehicleType

   worth = 0

   if VEHICLE_TYPE_UNKNOWN == vehicleType:
	   sendMsgAll("DEBUG: Destroyed vehicle " + rootVehicle.templateName + " - not in 21cw_constants!")
	   write("DEBUG: Destroyed vehicle " + rootVehicle.templateName + " - not in 21cw_constants! " + victim.getName())

   # if getDamage(vehicle) == 0:
   args = WORTHSCHEMA[vehicleType]
   worth = args[0]

   if debug: sendMsgAll("DEBUG: worth " + str(worth))

   return worth
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# Calculate a players worth - version 2
# ------------------------------------------------------------------------
def GetPlayerWorth2( victim, attacker ):
   victim_vehicle = victim.getVehicle()
   victim_rootVehicle = getRootParent(victim_vehicle)
   victim_vehicleType = getVehicleType(victim_rootVehicle.templateName)

   attacker_vehicle = attacker.getVehicle()
   attacker_rootVehicle = getRootParent(attacker_vehicle)
   attacker_vehicleType = getVehicleType(attacker_rootVehicle.templateName)

   #for debugging purposes
   if debug: sendMsgAll("DEBUG: Destroyed vehicle " + victim_rootVehicle.templateName + " - type: " + str(vehicleType))


   if VEHICLE_TYPE_UNKNOWN == victim_vehicleType:
	   sendMsgAll("DEBUG: Destroyed vehicle " + victim_rootVehicle.templateName + " - not in 21cw_constants!")
	   write("DEBUG: Destroyed vehicle " + victim_rootVehicle.templateName + " - not in 21cw_constants! " + victim.getName())

   if VEHICLE_TYPE_UNKNOWN == attacker_vehicleType:
	   sendMsgAll("DEBUG: attacking vehicle " + attacker_rootVehicle.templateName + " - not in 21cw_constants!")
	   write("DEBUG: attacking vehicle " + attacker_rootVehicle.templateName + " - not in 21cw_constants! " + attacker.getName())

   victim_worth = 0
   if getDamage(victim_vehicle) == 0:
	   victim_args = WORTHSCHEMA2[victom_vehicleType]
	   victim_worth = victim_args[0]

   attacker_args = WORTHSCHEMA2[attacker_vehicleType]
   attacker_worth = attacker_args[0]

   if debug: sendMsgAll("DEBUG: victim_worth " + str(victim_worth))
   if debug: sendMsgAll("DEBUG: attacker_worth " + str(attacker_worth))

   worth=0
   if victim_worth > attacker_worth:
	   worth=victim_worth - attacker_worth
   elif victim_worth > 0:
	   # at least one point for destroyed assets
	   worth=1


   return worth
# ------------------------------------------------------------------------
